// Ejemplo FreeImage 4
// Leer metadatos
// Abril 2023, Salvador Pozo, Con Clase

#include <iostream>
#include <iomanip>
#include <Freeimage.h>
#include <cstring>

void MostrarMetadatos(FREE_IMAGE_MDMODEL model, FIBITMAP *dib);
const char *TipoTag(FREE_IMAGE_MDTYPE);

int main(int argc, char**argv)
{
    FIBITMAP* dib;
	char *lpszPathName;
    FREE_IMAGE_FORMAT fif = FIF_UNKNOWN;
    FITAG *tag;
    unsigned short int fecha[] = {2023, 5, 22};

	if(argc < 2) {
        printf("Usar: FreeImage004 <fichero>\n");
        return 0;
	}
	lpszPathName = argv[1];

	fif = FreeImage_GetFileType(lpszPathName, 0);
	if(fif == FIF_UNKNOWN) {
		fif = FreeImage_GetFIFFromFilename(lpszPathName);
	}
	if((fif != FIF_UNKNOWN) && FreeImage_FIFSupportsReading(fif)) {
		dib = FreeImage_Load(fif, lpszPathName, 0);
		std::cout << "Fichero: " << lpszPathName << std::endl;

		// Insertar un comentario:
		FreeImage_SetMetadataKeyValue(FIMD_COMMENTS, dib, "Autor", "Fulanito de tal");
		tag = FreeImage_CreateTag();
		//FreeImage_SetTagKey(tag, "Fecha"); // Opcional, ya que es necesario indicarlo en FreeImage_SetMetadata
		FreeImage_SetTagType(tag, FIDT_SHORT);
		FreeImage_SetTagCount(tag, 3);
		FreeImage_SetTagLength(tag, 3*sizeof(unsigned short int));
		FreeImage_SetTagValue(tag, (const void*)fecha);
		FreeImage_SetMetadata(FIMD_COMMENTS, dib, "Fecha", tag);
        FreeImage_DeleteTag(tag);

		std::cout << "Metadatos de COMMENTS" << std::endl;
        std::cout << "------------" << std::endl;
		MostrarMetadatos(FIMD_COMMENTS, dib);
		std::cout << "Metadatos de MAIN" << std::endl;
        std::cout << "------------" << std::endl;
		MostrarMetadatos(FIMD_EXIF_MAIN, dib);
		std::cout << "Metadatos de EXIF" << std::endl;
        std::cout << "------------" << std::endl;
		MostrarMetadatos(FIMD_EXIF_EXIF, dib);
		std::cout << "Metadatos de GPS" << std::endl;
        std::cout << "------------" << std::endl;
		MostrarMetadatos(FIMD_EXIF_GPS, dib);
		std::cout << "Metadatos de MAKERNOTE" << std::endl;
        std::cout << "------------" << std::endl;
		MostrarMetadatos(FIMD_EXIF_MAKERNOTE, dib);
		std::cout << "Metadatos de INTEROP" << std::endl;
        std::cout << "------------" << std::endl;
		MostrarMetadatos(FIMD_EXIF_INTEROP, dib);
		std::cout << "Metadatos de IPTC" << std::endl;
        std::cout << "------------" << std::endl;
		MostrarMetadatos(FIMD_IPTC, dib);
		std::cout << "Metadatos de XMP" << std::endl;
        std::cout << "------------" << std::endl;
		MostrarMetadatos(FIMD_XMP, dib);
		std::cout << "Metadatos de ANIMATION" << std::endl;
        std::cout << "------------" << std::endl;
		MostrarMetadatos(FIMD_ANIMATION, dib);
		std::cout << "Metadatos de CUSTOM" << std::endl;
        std::cout << "------------" << std::endl;
		MostrarMetadatos(FIMD_CUSTOM, dib);
		std::cout << "Metadatos de EXIF_RAW" << std::endl;
        std::cout << "------------" << std::endl;
		MostrarMetadatos(FIMD_EXIF_RAW, dib);

		FreeImage_Save(FIF_PNG, dib, "modificado.png", PNG_Z_BEST_COMPRESSION | PNG_INTERLACED);
    }
	FreeImage_Unload(dib);

    return 0;
}

void MostrarMetadatos(FREE_IMAGE_MDMODEL model, FIBITMAP *dib) {
    FIMETADATA *handle;
    FITAG *tag;
    FREE_IMAGE_MDTYPE tipo;

    handle = FreeImage_FindFirstMetadata(model, dib, &tag);
    if(NULL != handle) {
        do {
            std::cout << "Key: " << FreeImage_GetTagKey(tag) << std::endl;
            if(FreeImage_GetTagDescription(tag))
                std::cout << "Descripcion: " << FreeImage_GetTagDescription(tag) << std::endl;
            std::cout << "ID: " << FreeImage_GetTagID(tag) << std::endl;
            tipo = FreeImage_GetTagType(tag);
            std::cout << "Tipo: " << TipoTag(tipo) << std::endl; // Tabla 12
            std::cout << "Cuenta: " << FreeImage_GetTagCount(tag) << std::endl;
            std::cout << "Longitud: " << FreeImage_GetTagLength(tag) << std::endl;
            std::cout << "Valor: ";
            std::cout << FreeImage_TagToString(model, tag) << std::endl;
            std::cout << std::endl;
            std::cout << "------------" << std::endl;
        } while(FreeImage_FindNextMetadata(handle, &tag));

        FreeImage_FindCloseMetadata(handle);
    }
}

const char *TipoTag(FREE_IMAGE_MDTYPE tipo) {
    switch(int(tipo)) {
        case 1: return "Entero sin signo 8 bits"; break;
        case 2: return "Cadena de caracteres ASCII terminada en cero"; break;
        case 3: return "Entero corto sin signo 16 bits"; break;
        case 4: return "Entero sin signo 32 bits"; break;
        case 5: return "Dos LONG sin signo: el primero es el numerador de una fraccion, el segundo el denominador"; break;
        case 6: return "Entero con signo 8 bits"; break;
        case 7: return "Un byte, depende del dato"; break;
        case 8: return "Entero con signo 16 bits"; break;
        case 9: return "Entero con signo 32 bits"; break;
        case 10: return "Dos LONG con signo: el primero es el numerador de una fraccion, el segundo el denominador"; break;
        case 11: return "Float de precision sencilla"; break;
        case 12: return "Float de precision doble"; break;
        case 13: return "LONG sin signo, pero se usa para almacenar desplazamientos"; break;
        case 14: return "RGBQUAD, 32 bits"; break;
        case 16: return "Entero sin signo de 64 bits"; break;
        case 17: return "Entero con signo de 64 bits"; break;
        case 18: return "LONG sin signo de 64 bits, pero se usa para almacenar desplazamientos"; break;
    }
    return "?";
}

